import 'package:kkbox_openapi/kkbox_openapi.dart';
import 'package:test/test.dart';

main() {
  group('A group of tests', () {
    KKBOXOpenAPI api;

    setUp(() {
      var clientID = "YOUR_CLIENT_ID";
      var clientSecret = "YOUR_CLIENT_SECRET";
      api = new KKBOXOpenAPI(clientID, clientSecret);
    });

    validateArtist(ArtistInfo artist) {
      expect(artist.id.length > 0, isTrue);
      expect(artist.name.length > 0, isTrue);
      expect(artist.url.length > 0, isTrue);
    }

    validateAlbum(AlbumInfo album) {
      expect(album.id.length > 0, isTrue);
      expect(album.name.length > 0, isTrue);
      expect(album.url.length > 0, isTrue);
    }

    validateTrack(TrackInfo track) {
      expect(track.id.length > 0, isTrue);
      expect(track.name.length > 0, isTrue);
      expect(track.url.length > 0, isTrue);
      expect(track.duration > 0, isTrue);
      if (track.album != null) {
        validateAlbum(track.album);
      }
    }

    validatePlaylist(PlaylistInfo playlist) {
      expect(playlist.id.length > 0, isTrue);
      expect(playlist.title.length > 0, isTrue);
      expect(playlist.url.length > 0, isTrue);
      if (playlist.tracks != null) {
        for (final track in playlist.tracks.tracks) {
          validateTrack(track);
        }
      }
    }

    test('Search Track', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var results = await api.search('Jay Chou', [SearchType.track]);
        expect(results.trackResults != null, isTrue);
        expect(results.artistResults == null, isTrue);
        expect(results.playlistResults == null, isTrue);
        expect(results.albumResults == null, isTrue);
        for (final track in results.trackResults.tracks) {
          validateTrack(track);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Search Artist', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var results = await api.search('周杰倫', [SearchType.artist]);
        expect(results.trackResults == null, isTrue);
        expect(results.artistResults != null, isTrue);
        expect(results.playlistResults == null, isTrue);
        expect(results.albumResults == null, isTrue);
        for (final artist in results.artistResults.artists) {
          validateArtist(artist);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Search Album', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var results = await api.search('Jay Chou', [SearchType.album]);
        expect(results.trackResults == null, isTrue);
        expect(results.artistResults == null, isTrue);
        expect(results.playlistResults == null, isTrue);
        expect(results.albumResults != null, isTrue);
        for (final album in results.albumResults.albums) {
          validateAlbum(album);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Search Playlist', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var results = await api.search('周杰倫', [SearchType.playlist]);
        expect(results.trackResults == null, isTrue);
        expect(results.artistResults == null, isTrue);
        expect(results.playlistResults != null, isTrue);
        expect(results.albumResults == null, isTrue);
        for (final playlist in results.playlistResults.playlists) {
          validatePlaylist(playlist);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Track', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var track = await api.fetchTrack('4kxvr3wPWkaL9_y3o_');
        validateTrack(track);
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Album', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var album = await api.fetchAlbum('WpTPGzNLeutVFHcFq6');
        validateAlbum(album);
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Tracks in Album', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var tracks = await api.fetchTracksInAlbum('WpTPGzNLeutVFHcFq6');
        for (final track in tracks.tracks) {
          validateTrack(track);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Artist', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var artist = await api.fetchArtist('8q3_xzjl89Yakn_7GB');
        validateArtist(artist);
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Artist Albums', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var albums = await api.fetchAlbumsOfArtist('8q3_xzjl89Yakn_7GB');
        for (final album in albums.albums) {
          validateAlbum(album);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Artist Top Tracks', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var tracks = await api.fetchTopTracksOfArtist('8q3_xzjl89Yakn_7GB');
        for (final track in tracks.tracks) {
          validateTrack(track);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Artist Related Artists', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var artists =
            await api.fetchRelatedArtistsOfArtist('8q3_xzjl89Yakn_7GB');
        for (final artist in artists.artists) {
          validateArtist(artist);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Playlist', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var playlist = await api.fetchPlaylist('OsyceCHOw-NvK5j6Vo');
        validatePlaylist(playlist);
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Tracks in Playlist', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var tracks = await api.fetchTracksInPlaylist('OsyceCHOw-NvK5j6Vo');
        for (final track in tracks.tracks) {
          validateTrack(track);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Featured Playlists', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var playlists = await api.fetchFeaturedPlaylists();
        for (final playlist in playlists.playlists) {
          validatePlaylist(playlist);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch New Hits Playlists', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var playlists = await api.fetchNewHitsPlaylists();
        for (final playlist in playlists.playlists) {
          validatePlaylist(playlist);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Featured Playlists Categories', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var categories = await api.fetchFeaturedPlaylistCategories();
        for (final category in categories.categories) {
          expect(category.id.length > 0, isTrue);
          expect(category.title.length > 0, isTrue);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Featured Playlists Category', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var category =
            await api.fetchFeaturedPlaylistsInCategory('CrBHGk1J1KEsQlPLoz');
        for (final playlist in category.playlists.playlists) {
          validatePlaylist(playlist);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Mood Stations', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var stations = await api.fetchMoodStations();
        for (final station in stations.stations) {
          expect(station.id.length > 0, isTrue);
          expect(station.name.length > 0, isTrue);
          expect(station.images.length > 0, isTrue);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Mood Station', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var station = await api.fetchTracksInMoodStation('4tmrBI125HMtMlO9OF');
        expect(station.id.length > 0, isTrue);
        expect(station.name.length > 0, isTrue);
        expect(station.images.length > 0, isTrue);
        for (final track in station.tracks.tracks) {
          validateTrack(track);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Genre Stations', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var stations = await api.fetchGenreStations();
        for (final station in stations.stations) {
          expect(station.id.length > 0, isTrue);
          expect(station.name.length > 0, isTrue);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Genre Station', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var station = await api.fetchTracksInGenreStation('9ZAb9rkyd3JFDBC0wF');
        expect(station.id.length > 0, isTrue);
        expect(station.name.length > 0, isTrue);
        for (final track in station.tracks.tracks) {
          validateTrack(track);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch New Released Album Categories', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var results = await api.fetchNewReleaseAlbumCategories();
        for (final category in results.categories) {
          expect(category.id.length > 0, isTrue);
          expect(category.title.length > 0, isTrue);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch New Released Album Category', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var category =
            await api.fetchNewReleasedAlbumInCategory('0pGAIGDf5SqYh_SyHr');
        expect(category.id.length > 0, isTrue);
        expect(category.title.length > 0, isTrue);
        for (final album in category.albums.albums) {
          validateAlbum(album);
        }
      } catch (err) {
        fail(err.toString());
      }
    });

    test('Fetch Charts', () async {
      try {
        await api.fetchAccessTokenByClientCredential();
        var charts = await api.fetchCharts();
        for (final playlist in charts.playlists) {
          validatePlaylist(playlist);
        }
      } catch (err) {
        fail(err.toString());
      }
    });
  });
}
